package by.zerg.layouteditor.gui;

import static by.zerg.layouteditor.helpers.Constants.*;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.io.File;
import java.util.List;

import javax.imageio.ImageIO;
import javax.swing.JPanel;

import org.apache.log4j.Logger;

import by.zerg.layouteditor.theme.Config;
import by.zerg.layouteditor.theme.Property;
import by.zerg.layouteditor.theme.Section;
import by.zerg.layouteditor.theme.Theme;

public class PreviewPanel extends JPanel {
	private static final Logger LOG = Logger.getLogger(PreviewPanel.class);

	Theme theme;

	public PreviewPanel(Theme theme) {
		this.theme = theme;
	}

	public PreviewPanel() {
		// TODO Auto-generated constructor stub
		setPreferredSize(new Dimension(PREVIEW_PANEL_DEFAULT_WIDTH,
				PREVIEW_PANEL_DEFAULT_HEIGHT));
	}

	@Override
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		try {

			Config config = theme.getConfig();
			Section tableSection = config.getSection(TABLE_SECTION);
			Section imagesSection = config.getSection(IMAGES_SECTION);
			Section holdem9maxSection = config.getSection(HOLDEM_9MAX_SECTION);
			try {
				String foreground = theme.getGraphicsFolder()
						+ FILE_PATH_SEPARATOR
						+ imagesSection.getPropertyValue(FOREGROUND_PROPERTY);
				if (LOG.isDebugEnabled()) {
					LOG.debug("loading foreground image: " + foreground);
				}
				Image foregroundImage = ImageIO.read(new File(foreground));
				if (LOG.isDebugEnabled()) {
					LOG.debug("foreground image: " + foreground + " loaded...");
				}
				g.drawImage(foregroundImage, 0, 0, getWidth(), getHeight(),
						null);
				if (LOG.isDebugEnabled()) {
					LOG.debug("foreground painted...");
				}
			} catch (Throwable t) {
				if (LOG.isDebugEnabled()) {
					LOG.error("unhandled exception while painting foreground...");
				}
			}
			try {
				String chatForeground = theme.getGraphicsFolder()
						+ "\\fg_L2.png";
				if (LOG.isDebugEnabled()) {
					LOG.debug("loading foreground image: " + chatForeground
							+ " for chat...");
				}
				Image foregroundImage = ImageIO.read(new File(chatForeground));
				if (LOG.isDebugEnabled()) {
					LOG.debug("foreground image: " + chatForeground
							+ " for chat loaded...");
				}
				g.drawImage(foregroundImage, 0, 0, getWidth(), getHeight(),
						null);
				if (LOG.isDebugEnabled()) {
					LOG.debug("chat foreground painted...");
				}

				Property chatEditRectProperty = tableSection
						.getProperty("chat_edit_rect");
				List<Integer> chatEditRectCoords = theme
						.getCoordinates(chatEditRectProperty);
				if (LOG.isDebugEnabled()) {
					LOG.debug("chatEditRectCoords = " + chatEditRectCoords);
				}
				g.setColor(Color.WHITE);
				int x1 = chatEditRectCoords.get(0);
				int y1 = chatEditRectCoords.get(1);
				int x2 = chatEditRectCoords.get(2);
				int y2 = chatEditRectCoords.get(3);
				g.fillRect(x1, y1, x2 - x1, y2 - y1);
				if (LOG.isDebugEnabled()) {
					LOG.debug("chat edit rect painted...");
				}

				int chatSpace = theme.getInteger(tableSection
						.getProperty("chat_space"));
				if (LOG.isDebugEnabled()) {
					LOG.debug("chat spase = " + chatSpace);
				}

				Property chatRectProperty = tableSection
						.getProperty("chat_rect");
				List<Integer> chatRectCoords = theme
						.getCoordinates(chatRectProperty);
				if (LOG.isDebugEnabled()) {
					LOG.debug("chatRectCoords = " + chatRectCoords);
				}
				// g.setColor(Color.WHITE);
				x1 = chatRectCoords.get(0);
				y1 = Math.max(chatRectCoords.get(1), y2
						+ (chatSpace > 0 ? chatSpace : 0));
				x2 = chatRectCoords.get(2);
				y2 = Math.max(chatRectCoords.get(3), y1);
				g.fillRect(x1, y1, x2 - x1, y2 - y1);
				if (LOG.isDebugEnabled()) {
					LOG.debug("chat rect painted...");
				}
			} catch (Throwable t) {
				if (LOG.isDebugEnabled()) {
					LOG.error("unhandled exception while painting chat foreground...");
				}
			}
			try {
				Property seatCoordsProperty = holdem9maxSection
						.getProperty(SEAT_COORDS_PROPERTY);
				List<Integer> seatCoords = theme
						.getCoordinates(seatCoordsProperty);
				if (LOG.isDebugEnabled()) {
					LOG.debug("seatCoords = " + seatCoords);
				}
				String seat = theme.getGraphicsFolder() + "\\label\\seat.bmp";
				if (LOG.isDebugEnabled()) {
					LOG.debug("loading seat image: " + seat);
				}
				Image seatImage = ImageIO.read(new File(seat));
				if (LOG.isDebugEnabled()) {
					LOG.debug("seat image: " + seat + " loaded...");
				}
				for (int i = 0; i < seatCoords.size(); i += 2) {
					int sw = seatImage.getWidth(null);
					int sh = seatImage.getHeight(null);
					int sx = seatCoords.get(i) - sw / 2;
					int sy = seatCoords.get(i + 1) - sh / 2;
					g.drawImage(seatImage, sx, sy, sw, sh, null);
					if (LOG.isDebugEnabled()) {
						LOG.debug("seat painted...");
					}
				}
				if (LOG.isDebugEnabled()) {
					LOG.debug("all seats were successfully painted...");
				}
			} catch (Throwable t) {
				if (LOG.isDebugEnabled()) {
					LOG.error("unhandled exception while painting seats...");
				}
			}
			try {
				Property buttonsCoordsProperty = tableSection
						.getProperty(BUTTON3_COORDS_PROPERTY);
				List<Integer> buttonsCoords = theme
						.getCoordinates(buttonsCoordsProperty);
				if (LOG.isDebugEnabled()) {
					LOG.debug("buttonsCoords = " + buttonsCoords);
				}
				String button = theme.getGraphicsFolder() + "\\ctrls\\btn.bmp";
				if (LOG.isDebugEnabled()) {
					LOG.debug("loading button image: " + button);
				}
				Image seatImage = ImageIO.read(new File(button));
				if (LOG.isDebugEnabled()) {
					LOG.debug("seat image: " + button + " loaded...");
				}
				for (int i = 0; i < buttonsCoords.size(); i += 2) {
					int sw = seatImage.getWidth(null);
					int sh = seatImage.getHeight(null);
					int sx1 = 0;
					int sy1 = 0;
					int sx2 = sw / 3;
					int sy2 = sh;
					int dx1 = buttonsCoords.get(i) - sw / 6;
					int dy1 = buttonsCoords.get(i + 1) - sh / 2;
					int dx2 = dx1 + sw / 3;
					int dy2 = dy1 + sh;

					g.drawImage(seatImage, dx1, dy1, dx2, dy2, sx1, sy1, sx2,
							sy2, null);
					if (LOG.isDebugEnabled()) {
						LOG.debug("button painted...");
					}
				}
				if (LOG.isDebugEnabled()) {
					LOG.debug("all buttons were successfully painted...");
				}
			} catch (Throwable t) {
				if (LOG.isDebugEnabled()) {
					LOG.error("unhandled exception while painting buttons...");
				}
			}
			if (LOG.isDebugEnabled()) {
				LOG.debug("all graphics were painted...");
			}
		} catch (Throwable t) {
			if (LOG.isDebugEnabled()) {
				LOG.error("unhandled exception while paining layout preview...");
			}
		}
	}

	public Theme getTheme() {
		return theme;
	}

	public void setTheme(Theme theme) {
		this.theme = theme;
		repaint();
	}
}
