package by.zerg.layouteditor.theme;

import static by.zerg.layouteditor.helpers.Constants.*;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.StringReader;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import org.apache.log4j.Logger;

public class Config {
	private static final Logger LOG = Logger.getLogger(Config.class);
	StringBuilder data;
	final String gx;
	Map<String, Section> sections;

	public Config(String gx) {
		this.gx = gx;
		data = new StringBuilder();
		sections = new LinkedHashMap<String, Section>();
	}

	public int execute(String cmd) {
		int exitValue = 1;
		try {
			Process p = Runtime.getRuntime().exec(cmd);
			BufferedReader processOutput = new BufferedReader(
					new InputStreamReader(p.getInputStream()));
			BufferedReader processError = new BufferedReader(
					new InputStreamReader(p.getErrorStream()));
			while (processOutput.readLine() != null
					|| processError.readLine() != null) {
			}
			processOutput.close();
			processError.close();
			exitValue = p.waitFor();
		} catch (Throwable t) {
			LOG.error("unhandled exception while executing binaries...", t);
		}
		if (LOG.isDebugEnabled()) {
			LOG.debug("result of executing command[ " + cmd + " ] is "
					+ exitValue);
		}
		return exitValue;
	}

	public int encrypt() throws IOException {
		String gzip = gx.replace(INI_FILE, GZ_FILE);

		GZIPOutputStream gzipOutputStream = new GZIPOutputStream(
				new FileOutputStream(gzip));
		PrintWriter writer = new PrintWriter(gzipOutputStream);

		BufferedReader reader = new BufferedReader(new StringReader(getData()));
		String line = null;
		while ((line = reader.readLine()) != null) {
			writer.println(line);
		}

		writer.flush();
		writer.close();
		reader.close();

		int result = execute(BIN_CRYPTER_EXE + SPACE + DOUBLE_QUOTES + gzip
				+ DOUBLE_QUOTES + SPACE + DOUBLE_QUOTES + gx + DOUBLE_QUOTES);
		new File(gzip).delete();
		return result;
	}

	public int decrypt() throws IOException {
		String gzip = gx.replace(INI_FILE, GZ_FILE);
		int result = execute(BIN_DECRYPTER_EXE + SPACE + DOUBLE_QUOTES + gx
				+ DOUBLE_QUOTES + SPACE + DOUBLE_QUOTES + gzip + DOUBLE_QUOTES);
		GZIPInputStream gzipInputStream = new GZIPInputStream(
				new FileInputStream(gzip));

		open(gzipInputStream);
		new File(gzip).delete();
		return result;
	}

	public int open() throws IOException {
		InputStream inputStream = new FileInputStream(gx);
		open(inputStream);
		return 0;
	}

	private void open(InputStream inputStream) throws IOException {
		data = new StringBuilder();
		sections = new LinkedHashMap<String, Section>();
		BufferedReader reader = new BufferedReader(new InputStreamReader(
				inputStream));
		String line = null;
		Section newSection = null;
		boolean flag = false;

		while ((line = reader.readLine()) != null) {
			data.append(line).append(NEW_LINE);
			line = line.trim();
			if (line.startsWith(SEMICOLUMN) || line.startsWith(SHARP)) {
				continue;
			} else if (line.startsWith(LEFT_SQUARE_BRACKET)
					&& line.endsWith(RIGHT_SQUARE_BRACKET)) {
				if (flag) {
					addSection(newSection);
				}
				flag = true;
				newSection = new Section(line.substring(1, line.length() - 1)
						.trim(), this);
			} else if (newSection != null && line.contains(EQUAL)) {
				int separator = line.indexOf(EQUAL);
				String propertyName = line.substring(0, separator).trim();
				String propertyValue = line.substring(separator + 1).trim();
				newSection.addProperty(new Property(propertyName,
						propertyValue, newSection));
			}
		}

		if (flag) {
			addSection(newSection);
		}
		inputStream.close();
	}

	@Override
	public String toString() {
		StringBuilder buff = new StringBuilder();
		for (Section section : sections.values()) {
			buff.append(section).append(NEW_LINE);
			for (Property property : section.getProperties().values()) {
				buff.append(property).append(NEW_LINE);
			}
			buff.append(NEW_LINE);
		}
		return buff.toString();
	}

	public Section getSection(String name) {
		return sections.get(name);
	}

	public void addSection(Section section) {
		sections.put(section.getName(), section);
	}

	public String getData() {
		return data.toString();
	}

	public void setData(String data) {
		this.data = new StringBuilder(data);
	}

	public String getGx() {
		return gx;
	}

	public Map<String, Section> getSections() {
		return sections;
	}

	public void setSections(Map<String, Section> sections) {
		this.sections = sections;
	}

	public void setData(StringBuilder data) {
		this.data = data;
	}
}
