package by.zerg.layouteditor.theme;

import static by.zerg.layouteditor.helpers.Constants.*;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

public class Loader {
	static List<Theme> defaultThemes;
	static List<Theme> customThemes;
	final List<Theme> themes;
	final String root;
	final String defaultThemesRoot;
	final String customThemesRoot;
	final Config defaultThemesConfig;
	final Config customThemesConfig;

	public Loader(String root) {
		this.root = root;
		defaultThemesRoot = root + THEMES_FOLDER;
		customThemesRoot = root + XTHEMES_FOLDER;
		themes = new ArrayList<Theme>();
		defaultThemesConfig = new Config(defaultThemesRoot + THEMES_INI_FILE);
		customThemesConfig = new Config(customThemesRoot + XTHEMES_INI_FILE);
	}

	public void load() throws IOException {
		// defaultThemes = new ArrayList<Theme>();
		// customThemes = new ArrayList<Theme>();
		themes.clear();
		// String defaultThemesRoot = root + THEMES_FOLDER;
		// String customThemesRoot = root + XTHEMES_FOLDER;
		// Config config = new Config(defaultThemesRoot + THEMES_INI_FILE);
		// config.decrypt();
		// addThemes(config, defaultThemes, defaultThemesRoot);
		defaultThemesConfig.decrypt();
		addThemes(defaultThemesConfig, Theme.DEFAULT_THEME);
		customThemesConfig.open();
		addThemes(customThemesConfig, Theme.CUSTOM_THEME);
		for (Theme theme : themes) {
			theme.setConfig(findConfig(theme));
		}
		// config = new Config(customThemesRoot + XTHEMES_INI_FILE);
		// config.open();
		// addThemes(config, customThemes, customThemesRoot);
	}

	//
	// public static List<Theme> getDefaultThemes() {
	// return defaultThemes;
	// }
	//
	// public static List<Theme> getCustomThemes() {
	// return customThemes;
	// }

	// private static void addThemes(Config config, List<Theme> themes,
	// String themesRoot) {
	// for (Section section : config.getSections().values()) {
	// String id = section.getName();
	// String name = section.getPropertyValue(NAME_PROPERTY);
	// String copyright = section.getPropertyValue(COPYRIGHT_PROPERTY);
	// String baseId = section.getPropertyValue(BASE_PROPERTY);
	// if (baseId == null && !id.equals(DEFAULT_THEME_ID)) {
	// baseId = DEFAULT_THEME_ID;
	// }
	// String folder = id.equals(DEFAULT_THEME_ID) ? root + GX_FOLDER
	// : themesRoot + FILE_PATH_SEPARATOR
	// + section.getPropertyValue(FOLDER_PROPERTY);
	// String deck = section.getPropertyValue(DECK_PROPERTY);
	// String feature = section.getPropertyValue(FEATURE_PROPERTY);
	// String lobby = section.getPropertyValue(LOBBY_PROPERTY);
	// String preview = section.getPropertyValue(PREVIEW_PROPERTY);
	// // for (Property property : section.getProperties().values()) {
	// // String propertyName = property.getName();
	// // String propertyValue = property.getValue();
	// // if (propertyName.equals("name")) {
	// // name = propertyValue;
	// // } else if (propertyName.equals("copyright")) {
	// // copyright = propertyValue;
	// // } else if (propertyName.equals("base")) {
	// // baseId = propertyValue;
	// // } else if (propertyName.equals("folder")) {
	// // folder = themesRoot + "\\" + propertyValue;
	// // } else if (propertyName.equals("deck")) {
	// // deck = propertyValue;
	// // } else if (propertyName.equals("feature")) {
	// // feature = propertyValue;
	// // } else if (propertyName.equals("lobby")) {
	// // lobby = propertyValue;
	// // } else if (propertyName.equals("preview")) {
	// // preview = propertyValue;
	// // }
	// // }
	// // if (id.equals("default@pokerstars.com")) {
	// // folder = root + "\\gx";
	// // }
	// themes.add(new Theme(id, name, copyright, baseId, folder, deck,
	// feature, lobby, preview));
	// }
	// for (Theme theme : themes) {
	// theme.setConfig(findConfig(theme));
	// }
	// }

	private void addThemes(Config config, int type) {
		for (Section section : config.getSections().values()) {
			String id = section.getName();
			String name = section.getPropertyValue(NAME_PROPERTY);
			String copyright = section.getPropertyValue(COPYRIGHT_PROPERTY);
			String baseId = section.getPropertyValue(BASE_PROPERTY);
			// if (baseId == null && !id.equals(DEFAULT_THEME_ID)) {
			// baseId = DEFAULT_THEME_ID;
			// }
			// String themesRoot = type == Theme.DEFAULT_THEME ?
			// defaultThemesRoot
			// : customThemesRoot;
			// String folder = id.equals(DEFAULT_THEME_ID) ? root + GX_FOLDER
			// : themesRoot + FILE_PATH_SEPARATOR
			// + section.getPropertyValue(FOLDER_PROPERTY);
			String folder = section.getPropertyValue(FOLDER_PROPERTY);
			String graphicsFolder = id.equals(DEFAULT_THEME_ID) ? root
					+ GX_FOLDER
					: (Theme.DEFAULT_THEME == type ? defaultThemesRoot
							: customThemesRoot) + FILE_PATH_SEPARATOR + folder;
			String deck = section.getPropertyValue(DECK_PROPERTY);
			String feature = section.getPropertyValue(FEATURE_PROPERTY);
			String lobby = section.getPropertyValue(LOBBY_PROPERTY);
			String preview = section.getPropertyValue(PREVIEW_PROPERTY);
			addTheme(new Theme(this, type, id, name, copyright, baseId, folder,
					graphicsFolder, deck, feature, lobby, preview));
		}
	}

	private Config findConfig(Theme theme) {
		String gx = defaultThemesRoot + FILE_PATH_SEPARATOR + theme.getFolder()
				+ GX_INI_FILE;
		return new File(gx).exists() ? new Config(gx) : findConfig(theme
				.getBaseTheme());
	}

	public Theme getTheme(String id) {
		Theme result = null;
		for (Theme theme : themes) {
			if (theme.getId().equals(id)) {
				result = theme;
				break;
			}
		}
		return result;
	}

	public void addTheme(Theme theme) {
		themes.add(theme);
	}

	public List<Theme> getThemes() {
		return themes;
	}

	public String getRoot() {
		return root;
	}

	public int launchClient() throws IOException, InterruptedException {
		Process p = Runtime.getRuntime().exec(
				root + FILE_PATH_SEPARATOR + POKERSTARS_EXE);
		// BufferedReader processOutput = new BufferedReader(
		// new InputStreamReader(p.getInputStream()));
		// BufferedReader processError = new BufferedReader(new
		// InputStreamReader(
		// p.getErrorStream()));
		// while (processOutput.readLine() != null
		// || processError.readLine() != null) {
		// }
		// processOutput.close();
		// processError.close();
		return p.waitFor();
	}
}
