package by.zerg.layouteditor.theme;

import static by.zerg.layouteditor.helpers.Constants.*;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

public class Theme {
	public static final int DEFAULT_THEME = 0;
	public static final int CUSTOM_THEME = 1;

	final Loader loader;
	final int type;
	final String id;
	final String name;
	final String copyright;
	final String baseId;
	final String folder;
	final String graphicsFolder;
	final String deck;
	final String feature;
	final String lobby;
	final String preview;
	Config config;

	public Theme(Loader loader, int type, String id, String name,
			String copyright, String baseId, String folder,
			String graphicsFolder, String deck, String feature, String lobby,
			String preview) {
		this.loader = loader;
		this.type = type;
		this.id = id;
		this.name = name;
		this.copyright = copyright;
		this.baseId = baseId;
		this.folder = folder;
		this.graphicsFolder = graphicsFolder;
		this.deck = deck;
		this.feature = feature;
		this.lobby = lobby;
		this.preview = preview;
	}

	@Override
	public String toString() {
		return id;
	}

	public boolean isDefault() {
		return DEFAULT_THEME == type;
	}

	public boolean isCustom() {
		return CUSTOM_THEME == type;
	}

	public Theme getBaseTheme() {
		if (id.equals(DEFAULT_THEME_ID)) {
			return null;
		}
		return loader.getTheme(baseId == null ? DEFAULT_THEME_ID : baseId);
	}

	public void open() throws IOException {
		config.decrypt();
	}

	public void save() throws IOException {
		config.encrypt();
	}

	public List<Integer> getCoordinates(Property property) {
		List<Integer> coords = new ArrayList<Integer>();
		StringTokenizer st = new StringTokenizer(property.getValue(), " ,");
		while (st.hasMoreTokens()) {
			coords.add(Integer.valueOf(st.nextToken()));
		}
		return coords;
	}

	public int getInteger(Property property) {
		return Integer.valueOf(property.getValue());
	}

	public Config getConfig() {
		return config;
	}

	public void setConfig(Config config) {
		this.config = config;
	}

	public Loader getLoader() {
		return loader;
	}

	public int getType() {
		return type;
	}

	public String getId() {
		return id;
	}

	public String getName() {
		return name;
	}

	public String getCopyright() {
		return copyright;
	}

	public String getBaseId() {
		return baseId;
	}

	public String getFolder() {
		return folder;
	}

	public String getGraphicsFolder() {
		return graphicsFolder;
	}

	public String getDeck() {
		return deck;
	}

	public String getFeature() {
		return feature;
	}

	public String getLobby() {
		return lobby;
	}

	public String getPreview() {
		return preview;
	}
}
